@@ -6,7 +6,6 @@ use crate::core::ribosome::guest_callback::entry_defs::EntryDefsResult;
 use crate::core::ribosome::real_ribosome::RealRibosome;
 use crate::core::ribosome::RibosomeT;
 
-use super::api::CellConductorApiT;
 use error::EntryDefStoreError;
 use error::EntryDefStoreResult;
 use holo_hash::*;
@@ -14,6 +13,8 @@ use holochain_serialized_bytes::prelude::*;
 use holochain_types::prelude::*;
 use std::collections::HashMap;
 
+use super::handle::ConductorHandleT;
+
 pub mod error;
 
 /// Get an [EntryDef] from the entry def store
@@ -22,13 +23,13 @@ pub(crate) async fn get_entry_def(
     entry_def_index: EntryDefIndex,
     zome: ZomeDef,
     dna_def: &DnaDefHashed,
-    conductor_api: &impl CellConductorApiT,
+    conductor_handle: &dyn ConductorHandleT,
 ) -> EntryDefStoreResult<Option<EntryDef>> {
     // Try to get the entry def from the entry def store
     let key = EntryDefBufferKey::new(zome, entry_def_index);
-    let entry_def = conductor_api.get_entry_def(&key);
+    let entry_def = conductor_handle.get_entry_def(&key);
     let dna_hash = dna_def.as_hash();
-    let dna_file = conductor_api
+    let dna_file = conductor_handle
         .get_dna(dna_hash)
         .ok_or_else(|| EntryDefStoreError::DnaFileMissing(dna_hash.clone()))?;
 
@@ -45,11 +46,11 @@ pub(crate) async fn get_entry_def_from_ids(
     zome_id: ZomeId,
     entry_def_index: EntryDefIndex,
     dna_def: &DnaDefHashed,
-    conductor_api: &impl CellConductorApiT,
+    conductor_handle: &dyn ConductorHandleT,
 ) -> EntryDefStoreResult<Option<EntryDef>> {
     match dna_def.zomes.get(zome_id.index()) {
         Some((_, zome)) => {
-            get_entry_def(entry_def_index, zome.clone(), dna_def, conductor_api).await
+            get_entry_def(entry_def_index, zome.clone(), dna_def, conductor_handle).await
         }
         None => Ok(None),
     }
