@@ -59,6 +59,7 @@ use crate::conductor::p2p_agent_store::query_peer_density;
 use crate::conductor::p2p_agent_store::P2pBatch;
 use crate::conductor::p2p_metrics::put_metric_datum;
 use crate::conductor::p2p_metrics::query_metrics;
+use crate::core::queue_consumer::QueueConsumerMap;
 use crate::core::ribosome::guest_callback::post_commit::PostCommitArgs;
 use crate::core::ribosome::real_ribosome::RealRibosome;
 use crate::core::workflow::ZomeCallResult;
@@ -74,10 +75,9 @@ use holochain_keystore::MetaLairClient;
 use holochain_p2p::event::HolochainP2pEvent;
 use holochain_p2p::event::HolochainP2pEvent::*;
 use holochain_p2p::DnaHashExt;
-
-use holochain_p2p::HolochainP2pCellT;
-use holochain_sqlite::db::DbKind;
-use holochain_state::host_fn_workspace::HostFnWorkspace;
+use holochain_p2p::HolochainP2pDnaT;
+use holochain_sqlite::conn::DbSyncStrategy;
+use holochain_state::host_fn_workspace::SourceChainWorkspace;
 use holochain_state::source_chain;
 use holochain_types::prelude::*;
 use kitsune_p2p::agent_store::AgentInfoSigned;
@@ -164,12 +164,15 @@ pub trait ConductorHandleT: Send + Sync {
     async fn call_zome_with_workspace(
         &self,
         invocation: ZomeCall,
-        workspace_lock: HostFnWorkspace,
+        workspace_lock: SourceChainWorkspace,
     ) -> ConductorApiResult<ZomeCallResult>;
 
     /// Get a Websocket port which will
     fn get_arbitrary_admin_websocket_port(&self) -> Option<u16>;
 
+    /// Get the running queue consumer workflows per [`DnaHash`] map.
+    fn get_queue_consumer_workflows(&self) -> QueueConsumerMap;
+
     /// Return the JoinHandle for all managed tasks, which when resolved will
     /// signal that the Conductor has completely shut down.
     ///
@@ -278,6 +281,12 @@ pub trait ConductorHandleT: Send + Sync {
         cell_id: &CellId,
     ) -> ConductorResult<HashSet<InstalledAppId>>;
 
+    /// Get the IDs of all active installed Apps which use this Dna
+    async fn list_running_apps_for_required_dna_hash(
+        &self,
+        dna_hash: &DnaHash,
+    ) -> ConductorResult<HashSet<InstalledAppId>>;
+
     /// Dump the cells state
     async fn dump_cell_state(&self, cell_id: &CellId) -> ConductorApiResult<String>;
 
@@ -310,24 +319,25 @@ pub trait ConductorHandleT: Send + Sync {
     /// Print the current setup in a machine readable way.
     fn print_setup(&self);
 
-    /// Retrieve the environment for this cell.
-    fn get_cell_env_readonly(&self, cell_id: &CellId) -> ConductorApiResult<EnvRead>;
-
     /// Manually remove some cells. Should only be used when handling errors in Cells,
     /// allowing individual Cells to be shut down.
     async fn remove_cells(&self, cell_ids: &[CellId]);
 
-    /// Retrieve the environment for this cell. FOR TESTING ONLY.
+    /// Retrieve the authored environment for this dna. FOR TESTING ONLY.
     #[cfg(any(test, feature = "test_utils"))]
-    fn get_cell_env(&self, cell_id: &CellId) -> ConductorApiResult<EnvWrite>;
+    fn get_authored_env(&self, cell_id: &DnaHash) -> ConductorApiResult<DbWrite<DbKindAuthored>>;
+
+    /// Retrieve the dht environment for this dna. FOR TESTING ONLY.
+    #[cfg(any(test, feature = "test_utils"))]
+    fn get_dht_env(&self, cell_id: &DnaHash) -> ConductorApiResult<DbWrite<DbKindDht>>;
 
     /// Retrieve the database for this cell. FOR TESTING ONLY.
     #[cfg(any(test, feature = "test_utils"))]
-    fn get_cache_env(&self, cell_id: &CellId) -> ConductorApiResult<EnvWrite>;
+    fn get_cache_env(&self, cell_id: &CellId) -> ConductorApiResult<DbWrite<DbKindCache>>;
 
     /// Retrieve the database for networking. FOR TESTING ONLY.
     #[cfg(any(test, feature = "test_utils"))]
-    fn get_p2p_env(&self, space: Arc<KitsuneSpace>) -> EnvWrite;
+    fn get_p2p_env(&self, space: Arc<KitsuneSpace>) -> DbWrite<DbKindP2pAgentStore>;
 
     /// Retrieve Senders for triggering workflows. FOR TESTING ONLY.
     #[cfg(any(test, feature = "test_utils"))]
@@ -445,13 +455,15 @@ pub struct ConductorHandleImpl<DS: DnaStore + 'static> {
     pub(super) root_env_dir: EnvironmentRootPath,
 
     /// The database for storing AgentInfoSigned
-    pub(super) p2p_env: Arc<parking_lot::Mutex<HashMap<Arc<KitsuneSpace>, EnvWrite>>>,
+    pub(super) p2p_env:
+        Arc<parking_lot::Mutex<HashMap<Arc<KitsuneSpace>, DbWrite<DbKindP2pAgentStore>>>>,
 
     /// The database for storing p2p MetricDatum(s)
-    pub(super) p2p_metrics_env: Arc<parking_lot::Mutex<HashMap<Arc<KitsuneSpace>, EnvWrite>>>,
+    pub(super) p2p_metrics_env:
+        Arc<parking_lot::Mutex<HashMap<Arc<KitsuneSpace>, DbWrite<DbKindP2pMetrics>>>>,
 
     /// Database sync level
-    pub(super) db_sync_level: DbSyncLevel,
+    pub(super) db_sync_strategy: DbSyncStrategy,
 
     /// The batch sender for writes to the p2p database.
     pub(super) p2p_batch_senders:
@@ -682,18 +694,59 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
             }
             HolochainP2pEvent::CallRemote { .. }
             | CountersigningAuthorityResponse { .. }
-            | Publish { .. }
             | GetValidationPackage { .. }
             | Get { .. }
             | GetMeta { .. }
             | GetLinks { .. }
             | GetAgentActivity { .. }
-            | ValidationReceiptReceived { .. }
-            | FetchOpData { .. } => {
+            | ValidationReceiptReceived { .. } => {
                 let cell_id = CellId::new(event.dna_hash().clone(), event.target_agents().clone());
                 let cell = self.cell_by_id(&cell_id)?;
                 cell.handle_holochain_p2p_event(event).await?;
             }
+            Publish {
+                dna_hash,
+                respond,
+                request_validation_receipt,
+                countersigning_session,
+                ops,
+                ..
+            } => {
+                async {
+                    let res = self
+                        .conductor
+                        .spaces
+                        .handle_publish(
+                            &dna_hash,
+                            request_validation_receipt,
+                            countersigning_session,
+                            ops,
+                        )
+                        .await
+                        .map_err(holochain_p2p::HolochainP2pError::other);
+                    respond.respond(Ok(async move { res }.boxed().into()));
+                }
+                .instrument(debug_span!("handle_publish"))
+                .await;
+            }
+            FetchOpData {
+                respond,
+                op_hashes,
+                dna_hash,
+                ..
+            } => {
+                async {
+                    let res = self
+                        .conductor
+                        .spaces
+                        .handle_fetch_op_data(&dna_hash, op_hashes)
+                        .await
+                        .map_err(holochain_p2p::HolochainP2pError::other);
+                    respond.respond(Ok(async move { res }.boxed().into()));
+                }
+                .instrument(debug_span!("handle_fetch_op_data"))
+                .await;
+            }
 
             // This event does not have a single Cell as a target, so we handle
             // it at the conductor level.
@@ -701,71 +754,21 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
             //       is meant for a single Cell, i.e. allow batching in general
             HolochainP2pEvent::QueryOpHashes {
                 dna_hash,
-                to_agents,
                 window,
                 max_ops,
                 include_limbo,
+                arc_set,
                 respond,
                 ..
             } => {
-                let mut hashes_and_times = Vec::with_capacity(to_agents.len());
+                let res = self
+                    .conductor
+                    .spaces
+                    .handle_query_op_hashes(&dna_hash, arc_set, window, max_ops, include_limbo)
+                    .await
+                    .map_err(holochain_p2p::HolochainP2pError::other);
 
-                // For each cell collect the hashes and times that fit within the
-                // agents interval and time window.
-                for (agent, arc_set) in to_agents {
-                    let cell_id = CellId::new(dna_hash.clone(), agent);
-                    let cell = self.cell_by_id(&cell_id)?;
-                    match cell
-                        .handle_query_op_hashes(arc_set, window.clone(), include_limbo)
-                        .await
-                    {
-                        Ok(t) => hashes_and_times.extend(t),
-                        Err(e) => {
-                            // If there's an error for any cell we want to fail the whole call.
-                            respond.respond(Ok(async move {
-                                Err(holochain_p2p::HolochainP2pError::other(e))
-                            }
-                            .boxed()
-                            .into()));
-                            return Ok(());
-                        }
-                    }
-                }
-                // Remove any duplicate hashes.
-                // Note vec must be sorted to remove duplicates.
-                hashes_and_times.sort_unstable_by(|a, b| a.0.cmp(&b.0));
-                hashes_and_times.dedup_by(|a, b| a.0 == b.0);
-
-                // Now sort by time so we can take up to max_ops.
-                hashes_and_times.sort_unstable_by_key(|(_, t)| *t);
-
-                // The start time bound if there is one.
-                let start = hashes_and_times.first().map(|(_, t)| *t);
-
-                // The end time bound if there is one.
-                let end = hashes_and_times
-                    .get(max_ops)
-                    .or_else(|| hashes_and_times.last())
-                    .map(|(_, t)| *t);
-
-                // Extract the hashes.
-                let hashes: Vec<_> = hashes_and_times
-                    .into_iter()
-                    .take(max_ops)
-                    .map(|(h, _)| h)
-                    .collect();
-
-                // The range is exclusive so we add one to the end.
-                let range = start.and_then(|s| {
-                    end.map(|e| {
-                        (
-                            hashes,
-                            s..(e.saturating_add(&std::time::Duration::from_millis(1))),
-                        )
-                    })
-                });
-
-                respond.respond(Ok(async move { Ok(range) }.boxed().into()));
+                respond.respond(Ok(async move { res }.boxed().into()));
             }
         }
         Ok(())
@@ -779,7 +782,7 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
     async fn call_zome_with_workspace(
         &self,
         call: ZomeCall,
-        workspace_lock: HostFnWorkspace,
+        workspace_lock: SourceChainWorkspace,
     ) -> ConductorApiResult<ZomeCallResult> {
         debug!(cell_id = ?call.cell_id);
         let cell = self.cell_by_id(&call.cell_id)?;
@@ -794,6 +797,10 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
         self.conductor.get_arbitrary_admin_websocket_port()
     }
 
+    fn get_queue_consumer_workflows(&self) -> QueueConsumerMap {
+        self.conductor.get_queue_consumer_workflows()
+    }
+
     fn shutdown(&self) {
         self.conductor.shutdown()
     }
@@ -821,18 +828,8 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
         let cell_id = CellId::new(dna_hash, agent_key);
         let cells = vec![(cell_id.clone(), membrane_proof)];
 
-        // Gather the directory and keystore to avoid holding the conductor read lock.
-        let root_env_dir = std::path::PathBuf::from(self.conductor.root_env_dir().clone());
-        let keystore = self.conductor.keystore().clone();
         // Run genesis on cells.
-        crate::conductor::conductor::genesis_cells(
-            root_env_dir,
-            keystore,
-            cells,
-            self.clone(),
-            self.db_sync_level,
-        )
-        .await?;
+        crate::conductor::conductor::genesis_cells(&self.conductor, cells, self.clone()).await?;
 
         let properties = properties.unwrap_or_else(|| ().into());
         let cell_id = self
@@ -851,19 +848,13 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
         installed_app_id: InstalledAppId,
         cell_data: Vec<(InstalledCell, Option<MembraneProof>)>,
     ) -> ConductorResult<()> {
-        // Gather the directory and keystore to avoid holding the conductor read lock.
-        let root_env_dir = std::path::PathBuf::from(self.conductor.root_env_dir().clone());
-        let keystore = self.conductor.keystore().clone();
-
         crate::conductor::conductor::genesis_cells(
-            root_env_dir,
-            keystore,
+            &self.conductor,
             cell_data
                 .iter()
                 .map(|(c, p)| (c.as_id().clone(), p.clone()))
                 .collect(),
             self.clone(),
-            self.db_sync_level,
         )
         .await?;
 
@@ -911,18 +902,8 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
             self.clone().register_dna(dna).await?;
         }
 
-        // Gather the directory and keystore to avoid holding the conductor read lock.
-        let root_env_dir = std::path::PathBuf::from(self.conductor.root_env_dir().clone());
-        let keystore = self.conductor.keystore().clone();
-
-        crate::conductor::conductor::genesis_cells(
-            root_env_dir,
-            keystore,
-            cells_to_create,
-            self.clone(),
-            self.db_sync_level,
-        )
-        .await?;
+        crate::conductor::conductor::genesis_cells(&self.conductor, cells_to_create, self.clone())
+            .await?;
 
         let roles = ops.role_assignments;
         let app = InstalledAppCommon::new(installed_app_id, agent_key, roles);
@@ -1103,9 +1084,19 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
         self.conductor.list_running_apps_for_cell_id(cell_id).await
     }
 
+    async fn list_running_apps_for_required_dna_hash(
+        &self,
+        dna_hash: &DnaHash,
+    ) -> ConductorResult<HashSet<InstalledAppId>> {
+        self.conductor
+            .list_running_apps_for_dna_hash(dna_hash)
+            .await
+    }
+
     async fn dump_cell_state(&self, cell_id: &CellId) -> ConductorApiResult<String> {
         let cell = self.conductor.cell_by_id(cell_id)?;
-        let arc = cell.env();
+        let authored_env = cell.authored_env();
+        let dht_env = cell.dht_env();
         let space = cell_id.dna_hash().to_kitsune();
         let p2p_env = self
             .p2p_env
@@ -1116,12 +1107,13 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
 
         let peer_dump = p2p_agent_store::dump_state(p2p_env.into(), Some(cell_id.clone())).await?;
         let source_chain_dump =
-            source_chain::dump_state(arc.clone().into(), cell_id.agent_pubkey().clone()).await?;
+            source_chain::dump_state(authored_env.clone().into(), cell_id.agent_pubkey().clone())
+                .await?;
 
         let out = JsonDump {
             peer_dump,
             source_chain_dump,
-            integration_dump: integration_dump(&arc.clone().into()).await?,
+            integration_dump: integration_dump(&dht_env.clone().into()).await?,
         };
         // Add summary
         let summary = out.to_string();
@@ -1134,8 +1126,10 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
         cell_id: &CellId,
         dht_ops_cursor: Option<u64>,
     ) -> ConductorApiResult<FullStateDump> {
-        let cell = self.conductor.cell_by_id(cell_id)?;
-        let arc = cell.env();
+        let authored_env = self
+            .conductor
+            .get_or_create_authored_env(cell_id.dna_hash())?;
+        let dht_env = self.conductor.get_or_create_dht_env(cell_id.dna_hash())?;
         let space = cell_id.dna_hash().to_kitsune();
         let p2p_env = self
             .p2p_env
@@ -1146,12 +1140,12 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
 
         let peer_dump = p2p_agent_store::dump_state(p2p_env.into(), Some(cell_id.clone())).await?;
         let source_chain_dump =
-            source_chain::dump_state(arc.clone().into(), cell_id.agent_pubkey().clone()).await?;
+            source_chain::dump_state(authored_env.into(), cell_id.agent_pubkey().clone()).await?;
 
         let out = FullStateDump {
             peer_dump,
             source_chain_dump,
-            integration_dump: full_integration_dump(&arc.clone().into(), dht_ops_cursor).await?,
+            integration_dump: full_integration_dump(&dht_env, dht_ops_cursor).await?,
         };
         Ok(out)
     }
@@ -1217,29 +1211,28 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
         self.conductor.print_setup()
     }
 
-    fn get_cell_env_readonly(&self, cell_id: &CellId) -> ConductorApiResult<EnvRead> {
-        let cell = self.cell_by_id(cell_id)?;
-        Ok(cell.env().clone().into())
-    }
-
     async fn remove_cells(&self, cell_ids: &[CellId]) {
         self.conductor.remove_cells(cell_ids.to_vec()).await
     }
 
     #[cfg(any(test, feature = "test_utils"))]
-    fn get_cell_env(&self, cell_id: &CellId) -> ConductorApiResult<EnvWrite> {
-        let cell = self.cell_by_id(cell_id)?;
-        Ok(cell.env().clone())
+    fn get_authored_env(&self, dna_hash: &DnaHash) -> ConductorApiResult<DbWrite<DbKindAuthored>> {
+        Ok(self.conductor.get_or_create_authored_env(dna_hash)?)
     }
 
     #[cfg(any(test, feature = "test_utils"))]
-    fn get_cache_env(&self, cell_id: &CellId) -> ConductorApiResult<EnvWrite> {
+    fn get_dht_env(&self, dna_hash: &DnaHash) -> ConductorApiResult<DbWrite<DbKindDht>> {
+        Ok(self.conductor.get_or_create_dht_env(dna_hash)?)
+    }
+
+    #[cfg(any(test, feature = "test_utils"))]
+    fn get_cache_env(&self, cell_id: &CellId) -> ConductorApiResult<DbWrite<DbKindCache>> {
         let cell = self.cell_by_id(cell_id)?;
         Ok(cell.cache().clone())
     }
 
     #[cfg(any(test, feature = "test_utils"))]
-    fn get_p2p_env(&self, space: Arc<KitsuneSpace>) -> EnvWrite {
+    fn get_p2p_env(&self, space: Arc<KitsuneSpace>) -> DbWrite<DbKindP2pAgentStore> {
         self.p2p_env(space)
     }
 
@@ -1398,9 +1391,9 @@ impl<DS: DnaStore + 'static> ConductorHandleImpl<DS> {
             .conductor
             .mark_pending_cells_as_joining()
             .into_iter()
-            .map(|(id, cell)| (id, cell.holochain_p2p_cell().clone()))
+            .map(|(id, cell)| (id, cell.holochain_p2p_dna().clone()))
             .map(|(cell_id, network)| async move {
-                match tokio::time::timeout(JOIN_NETWORK_TIMEOUT, network.join()).await {
+                match tokio::time::timeout(JOIN_NETWORK_TIMEOUT, network.join(cell_id.agent_pubkey().clone())).await {
                     Ok(Err(e)) => {
                         tracing::info!(error = ?e, cell_id = ?cell_id, "Error while trying to join the network");
                         Err(cell_id)
@@ -1436,19 +1429,20 @@ impl<DS: DnaStore + 'static> ConductorHandleImpl<DS> {
         cell_ids
     }
 
-    pub(super) fn p2p_env(&self, space: Arc<KitsuneSpace>) -> EnvWrite {
+    pub(super) fn p2p_env(&self, space: Arc<KitsuneSpace>) -> DbWrite<DbKindP2pAgentStore> {
         let mut p2p_env = self.p2p_env.lock();
-        let db_sync_level = self.db_sync_level;
+        let db_sync_strategy = self.db_sync_strategy;
         p2p_env
             .entry(space.clone())
             .or_insert_with(move || {
                 let root_env_dir = self.root_env_dir.as_ref();
-                let keystore = self.keystore.clone();
-                EnvWrite::open_with_sync_level(
+                DbWrite::open_with_sync_level(
                     root_env_dir,
-                    DbKind::P2pAgentStore(space),
-                    keystore,
-                    db_sync_level,
+                    DbKindP2pAgentStore(space),
+                    match db_sync_strategy {
+                        DbSyncStrategy::Fast => DbSyncLevel::Off,
+                        DbSyncStrategy::Resilient => DbSyncLevel::Normal,
+                    },
                 )
                 .expect("failed to open p2p_agent_store database")
             })
@@ -1471,19 +1465,20 @@ impl<DS: DnaStore + 'static> ConductorHandleImpl<DS> {
             .clone()
     }
 
-    pub(super) fn p2p_metrics_env(&self, space: Arc<KitsuneSpace>) -> EnvWrite {
+    pub(super) fn p2p_metrics_env(&self, space: Arc<KitsuneSpace>) -> DbWrite<DbKindP2pMetrics> {
         let mut p2p_metrics_env = self.p2p_metrics_env.lock();
-        let db_sync_level = self.db_sync_level;
+        let db_sync_strategy = self.db_sync_strategy;
         p2p_metrics_env
             .entry(space.clone())
             .or_insert_with(move || {
                 let root_env_dir = self.root_env_dir.as_ref();
-                let keystore = self.keystore.clone();
-                EnvWrite::open_with_sync_level(
+                DbWrite::open_with_sync_level(
                     root_env_dir,
-                    DbKind::P2pMetrics(space),
-                    keystore,
-                    db_sync_level,
+                    DbKindP2pMetrics(space),
+                    match db_sync_strategy {
+                        DbSyncStrategy::Fast => DbSyncLevel::Off,
+                        DbSyncStrategy::Resilient => DbSyncLevel::Normal,
+                    },
                 )
                 .expect("failed to open p2p_metrics database")
             })
