@@ -2,15 +2,17 @@
 
 use super::error::WorkflowResult;
 use super::sys_validation_workflow::counterfeit_check;
-use crate::core::queue_consumer::TriggerSender;
+use crate::{
+    conductor::{conductor::RwShare, space::Space},
+    core::queue_consumer::TriggerSender,
+};
 use holo_hash::DhtOpHash;
 use holochain_sqlite::error::DatabaseResult;
 use holochain_sqlite::prelude::*;
 use holochain_state::prelude::*;
 use holochain_types::dht_op::DhtOp;
 use holochain_types::prelude::*;
-use once_cell::sync::Lazy;
-use std::collections::{HashMap, HashSet};
+use std::{collections::HashSet, sync::Arc};
 use tracing::instrument;
 
 #[cfg(test)]
@@ -19,12 +21,17 @@ mod test;
 type InOpBatchSnd = tokio::sync::oneshot::Sender<WorkflowResult<()>>;
 type InOpBatchRcv = tokio::sync::oneshot::Receiver<WorkflowResult<()>>;
 
+#[derive(Debug)]
 struct InOpBatchEntry {
     snd: InOpBatchSnd,
     request_validation_receipt: bool,
     ops: Vec<(DhtOpHash, DhtOp)>,
 }
 
+/// A batch of incoming ops memory.
+#[derive(Clone)]
+pub struct IncomingOpsBatch(RwShare<InOpBatch>);
+
 struct InOpBatch {
     is_running: bool,
     pending: Vec<InOpBatchEntry>,
@@ -39,13 +46,16 @@ impl Default for InOpBatch {
     }
 }
 
-static IN_OP_BATCH: Lazy<parking_lot::Mutex<HashMap<DbKind, InOpBatch>>> =
-    Lazy::new(|| parking_lot::Mutex::new(HashMap::new()));
+impl Default for IncomingOpsBatch {
+    fn default() -> Self {
+        Self(RwShare::new(InOpBatch::default()))
+    }
+}
 
 /// if result.0.is_none() -- we queued it to send later
 /// if result.0.is_some() -- the batch should be run now
 fn batch_check_insert(
-    kind: DbKind,
+    batch: &IncomingOpsBatch,
     request_validation_receipt: bool,
     ops: Vec<(DhtOpHash, DhtOp)>,
 ) -> (Option<Vec<InOpBatchEntry>>, InOpBatchRcv) {
@@ -55,35 +65,35 @@ fn batch_check_insert(
         request_validation_receipt,
         ops,
     };
-    let mut lock = IN_OP_BATCH.lock();
-    let batch = lock.entry(kind).or_insert_with(InOpBatch::default);
-    if batch.is_running {
-        // there is already a batch running, just queue this
-        batch.pending.push(entry);
-        (None, rcv)
-    } else {
-        // no batch running, run this (and assert we never collect straglers
-        assert!(batch.pending.is_empty());
-        batch.is_running = true;
-        (Some(vec![entry]), rcv)
-    }
+    batch.0.share_mut(|batch| {
+        if batch.is_running {
+            // there is already a batch running, just queue this
+            batch.pending.push(entry);
+            (None, rcv)
+        } else {
+            // no batch running, run this (and assert we never collect straglers
+            assert!(batch.pending.is_empty());
+            batch.is_running = true;
+            (Some(vec![entry]), rcv)
+        }
+    })
 }
 
 /// if result.is_none() -- we are done, end the loop for now
 /// if result.is_some() -- we got more items to process
-fn batch_check_end(kind: DbKind) -> Option<Vec<InOpBatchEntry>> {
-    let mut lock = IN_OP_BATCH.lock();
-    let batch = lock.entry(kind).or_insert_with(InOpBatch::default);
-    assert!(batch.is_running);
-    let out: Vec<InOpBatchEntry> = batch.pending.drain(..).collect();
-    if out.is_empty() {
-        // pending was empty, we can end the loop for now
-        batch.is_running = false;
-        None
-    } else {
-        // we have some more pending, continue the running loop
-        Some(out)
-    }
+fn batch_check_end(batch: &IncomingOpsBatch) -> Option<Vec<InOpBatchEntry>> {
+    batch.0.share_mut(|batch| {
+        assert!(batch.is_running);
+        let out: Vec<InOpBatchEntry> = batch.pending.drain(..).collect();
+        if out.is_empty() {
+            // pending was empty, we can end the loop for now
+            batch.is_running = false;
+            None
+        } else {
+            // we have some more pending, continue the running loop
+            Some(out)
+        }
+    })
 }
 
 fn batch_process_entry(
@@ -113,23 +123,24 @@ fn batch_process_entry(
 #[derive(Default, Clone)]
 pub struct IncomingOpHashes(Arc<parking_lot::Mutex<HashSet<DhtOpHash>>>);
 
-#[instrument(skip(vault, sys_validation_trigger, ops, incoming_op_hashes))]
+#[instrument(skip(space, sys_validation_trigger, ops))]
 pub async fn incoming_dht_ops_workflow(
-    vault: &EnvWrite,
-    incoming_op_hashes: Option<&IncomingOpHashes>,
+    space: &Space,
     sys_validation_trigger: TriggerSender,
     mut ops: Vec<(holo_hash::DhtOpHash, holochain_types::dht_op::DhtOp)>,
     request_validation_receipt: bool,
 ) -> WorkflowResult<()> {
+    let Space {
+        incoming_op_hashes,
+        incoming_ops_batch,
+        dht_env,
+        ..
+    } = space;
     let mut filter_ops = Vec::new();
-    let mut hashes_to_remove = if incoming_op_hashes.is_some() {
-        Vec::with_capacity(ops.len())
-    } else {
-        Vec::with_capacity(0)
-    };
+    let mut hashes_to_remove = Vec::with_capacity(ops.len());
 
     // Filter out ops that are already being tracked, so we don't do duplicate work
-    if let Some(incoming_op_hashes) = &incoming_op_hashes {
+    {
         let mut set = incoming_op_hashes.0.lock();
         let mut o = Vec::with_capacity(ops.len());
         for (hash, op) in ops {
@@ -149,7 +160,7 @@ pub async fn incoming_dht_ops_workflow(
     for (hash, op) in ops {
         // It's cheaper to check if the op exists before trying
         // to check the signature or open a write transaction.
-        if !op_exists(vault, hash.clone()).await? {
+        if !op_exists(dht_env, hash.clone()).await? {
             match should_keep(&op).await {
                 Ok(()) => filter_ops.push((hash, op)),
                 Err(e) => {
@@ -163,18 +174,18 @@ pub async fn incoming_dht_ops_workflow(
         }
     }
 
-    let kind = vault.kind().clone();
     let (mut maybe_batch, rcv) =
-        batch_check_insert(kind.clone(), request_validation_receipt, filter_ops);
+        batch_check_insert(incoming_ops_batch, request_validation_receipt, filter_ops);
 
-    let vault = vault.clone();
+    let incoming_ops_batch = incoming_ops_batch.clone();
+    let dht_env = dht_env.clone();
     if maybe_batch.is_some() {
         // there was no already running batch task, so spawn one:
         tokio::task::spawn(async move {
             while let Some(entries) = maybe_batch {
                 let senders = Arc::new(parking_lot::Mutex::new(Vec::new()));
                 let senders2 = senders.clone();
-                if let Err(err) = vault
+                if let Err(err) = dht_env
                     .async_commit(move |txn| {
                         for entry in entries {
                             let InOpBatchEntry {
@@ -203,7 +214,7 @@ pub async fn incoming_dht_ops_workflow(
                 // trigger validation of queued ops
                 sys_validation_trigger.trigger();
 
-                maybe_batch = batch_check_end(kind.clone());
+                maybe_batch = batch_check_end(&incoming_ops_batch);
             }
         });
     }
@@ -212,7 +223,7 @@ pub async fn incoming_dht_ops_workflow(
         .await
         .map_err(|_| super::error::WorkflowError::RecvError)?;
 
-    if let Some(incoming_op_hashes) = &incoming_op_hashes {
+    {
         let mut set = incoming_op_hashes.0.lock();
         for hash in hashes_to_remove {
             set.remove(&hash);
@@ -236,7 +247,7 @@ fn add_to_pending(
 ) -> StateMutationResult<()> {
     for op in ops {
         let op_hash = op.as_hash().clone();
-        insert_op(txn, op, false)?;
+        insert_op(txn, op)?;
         set_require_receipt(txn, op_hash, request_validation_receipt)?;
     }
     StateMutationResult::Ok(())
@@ -260,7 +271,7 @@ fn op_exists_inner(txn: &rusqlite::Transaction<'_>, hash: &DhtOpHash) -> Databas
     )?)
 }
 
-pub async fn op_exists(vault: &EnvWrite, hash: DhtOpHash) -> DatabaseResult<bool> {
+pub async fn op_exists(vault: &DbWrite<DbKindDht>, hash: DhtOpHash) -> DatabaseResult<bool> {
     vault
         .async_reader(move |txn| op_exists_inner(&txn, &hash))
         .await
